package wat

import (
	"fmt"
	"strconv"
	"strings"

	"github.com/go-irc/irc"
)

type BotGameConfig map[string][]string

type WatIntegrationConfig struct {
	BotHosts []string
	BotGames BotGameConfig
}

type WatIntegration struct {
	bot *WatBot
	db  *WatDb
	c   *WatIntegrationConfig
}

func NewWatIntegration(bot *WatBot, db *WatDb, c *WatIntegrationConfig) *WatIntegration {
	return &WatIntegration{bot, db, c}
}

func (w *WatIntegration) Bot(m *irc.Message) (bool, []string) {
	isBot := w.bot.Allowed(m.Prefix.Host, w.c.BotHosts)
	var games []string
	if isBot {
		for b, g := range w.c.BotGames {
			if b == m.Prefix.Name {
				games = g
				break
			}
		}
	}
	return isBot, games
}

func (w *WatIntegration) HandleIntegration(m *irc.Message, msgargs []string, games []string) bool {

	// handles a message "Top finishers: (nick1: 1300) (nick2: 1200)" from an authorized Jeopardy game bot
	if msgargs[0] == "Top" && msgargs[1] == "finishers:" && w.bot.Allowed("jeopardy", games) {
		w.Jeopardy(m, msgargs)
		return true
	}

	return false
}

func (w *WatIntegration) Jeopardy(m *irc.Message, msgargs []string) {
	// hey, I avoided regex!
	finisherPrizes := strings.Split(strings.Replace(strings.Replace(strings.Replace(strings.Replace(strings.Join(msgargs[2:], " "), ") (", ";", -1), ": ", ":", -1), "(", "", 1), ")", "", 1), ";")
	fmt.Printf("Processing Jeopardy: %s\n", finisherPrizes)
	for _, pair := range finisherPrizes {
		nameCoinPair := strings.Split(pair, ":")
		coins, err := strconv.ParseUint(nameCoinPair[1], 10, 64)
		if err != nil {
			fmt.Printf("Invalid coins, cannot process pair for cashout: %s.\n", nameCoinPair)
			continue
		}
		name := nameCoinPair[0]
		// Jeopardy prizes are quite a lot of $$$, make it a bit more sane
		coins = coins / 40
		// name   = we assume the Jeopardy player name to match a Watbot player name
		// host   = we could use some WHO logic to find the host, but assuming nickname lookup to be sufficient here
		// create = based on the above, maybe rather not create Watbot players based on only a nick?
		//          but it expects someone to have played with Watbot before to be eligible for Jeopardy cashout ..
		player := w.db.User(name, "", false)
		if player.Nick == "" {
			fmt.Printf("Player %s does not exist in Watbot, skipping cashout.\n", name)
			continue
		} else {
			w.bot.reply(m, fmt.Sprintf("smartass %s, gave u %d :)", player.Nick, coins))
			player.Coins += coins
			w.db.Update(player)
		}
	}
}
